#!/usr/bin/perl
use File::Find;
use File::Basename;
#use strict;

my @directories_to_search = ( "kernel", "driver", "interface", "lapack", "lapack-3.1.1" );

my %symbol = ();

my @arnms = ();

sub getarnms()
{
    my @ary=();
    my $fnm = glob("libgoto2_*.a");
    return @ary unless -f $fnm ;
    open(FD, "ar t $fnm |");
    while(<FD>){
	chop($_);
        push @ary, $_;
    }
    close(FD);
    return @ary;
}

sub symbol_find($)
{
    return unless -f $_ ;
    return unless /\.o$/;

    my $fnm = $_;

    my $basenm = basename($fnm);

    unless ( grep {$_ eq $basenm} @arnms) {
        warn "not found $fnm in archive library.";
	return;
    }

    open(FD, "nm \"$fnm\" |") or die($File::Find::name);
    while(<FD>){
	next unless / [TDW] /;
	my @funcname = split(/[\s\r\n\t]+/,$_);
	if ( $#funcname >= 2 ){	
	    my $fc = $funcname[2];
	    $fc =~ s/^_//;
	    $symbol{$fc}=$File::Find::name;		
	}
    }
    close(FD);
}

sub symbollist_to_array($)
{
    my $fnm = shift;
    open(FD, $fnm) or die($fnm);
    my @list = ();
    while(<FD>){
	s/[\r\n]//g;
	push @list, $_;
    }
    close(FD);

    return @list;
}

sub symbollist_trim($)
{
    my $master = shift;
    my @list=();
    foreach (@$master){
	my $fnm = $_;
	$fnm =~ s/_$//;
	push @list,$fnm;
    }
    return @list;
}

@arnms = getarnms();
find ( {wanted=>\&symbol_find},  @directories_to_search );

my @blaslist   = symbollist_to_array("BLASLIST");
my @lapacklist = symbollist_to_array("LAPACKLIST");


my @blaslist_trim    = symbollist_trim(\@blaslist);
my @lapacklist_trim  = symbollist_trim(\@lapacklist);

my @blasobj=();
my @lapackobj=();



FN:
foreach my $funm (sort(keys(%symbol))){

    if ( $funm =~ /^blas_/){
	push @blasobj, $symbol{$funm};
	next FN;
    }
    if ( $funm =~ /^cblas_/){
	push @blasobj, $symbol{$funm};
	next FN;
    }
	
    my @m=();


    @m = grep( /^$funm[_]{0,1}$/, @blaslist);
    if ( $#m >= 0 ){ 
	push @blasobj, $symbol{$funm};
	next FN;
    }
    @m = grep(/^$funm[_]{0,1}$/,@lapacklist);
    if ($#m >= 0 ){
	push @lapackobj, $symbol{$funm};
	next FN;
    }
    {
	my $nm=$funm;
	$nm =~ s/.$//;
	while ( length($nm) > 3 ){
	    @m = grep(/^$nm$/, @blaslist);
	    if ($#m >= 0 ){	
		push @blasobj, $symbol{$funm};
		next FN;
	    }
	    @m = grep(/^$nm$/, @lapacklist);
	    if ($#m >= 0 ){
		push @lapackobj, $symbol{$funm};
		next FN; 
	    }
	    $nm =~ s/.$//g;
	}
    }
    {
	my $nm=$funm;
	$nm =~ s/_$//g;
	@m = grep(/^$nm$/,@blaslist_trim);
	if ( $#m >= 0 ){ 
	    push @blasobj, $symbol{$funm};
	    next FN;
	}
	@m = grep(/^$nm$/,@lapacklist_trim);
	if ($#m >= 0 ){
	    push @lapackobj, $symbol{$funm};
	    next FN;
	}
    }
    # other
    push @blasobj, $symbol{$funm};
}


sub writeobjs($$)
{
    my $filename = shift;
    my $array    = shift;

    my %seen ={}; 
    my @unique = grep !$seen{$_}++, @$array;

    open (FD, ">".$filename) or die($filename);
    foreach (sort(@unique)){
	print FD $_,"\n";
    }
    close(FD);
}
	
 
writeobjs("blasobj",\@blasobj);

writeobjs("lapackobj",\@lapackobj);

    

